using System.Collections.Generic;
using UnityEngine;
using System;
using Battlehub.RTSaveLoad;
using UnityEngine.Events;

namespace Ruvik
{
    class PH : MVRScript
    {
        static Keybind activationKb, switchMotionKb, switchAxisKb;
        Dictionary<string, Keybind> keyBindPairs = new Dictionary<string, Keybind>();
        static bool isRecording;
        string keyString = string.Empty;
        public static JSONStorableFloat sensetivity, rotationSpeed;
        public static JSONStorableBool focusSelectedBones, adjustPosition, alignUpDown;

        // This method will be called by Keybindings when it is ready.
        public void OnBindingsListRequested(List<object> bindings)
        {
            bindings.Add(new JSONStorableAction("activate/deactive UI", () => PHUtils.menuUIActive  = !PHUtils.menuUIActive));
            bindings.Add(new JSONStorableAction("clear selected bones", () => PHUtils.ClearPressedButtons()));
            bindings.Add(new JSONStorableAction("ik link", () => PHUtils.LinkAll_method()));
            bindings.Add(new JSONStorableAction("reset linking state", () => PHUtils.ResetLink_method()));
            bindings.Add(new JSONStorableAction("reset pose", () => PHUtils.ResetPose_method()));
            bindings.Add(new JSONStorableAction("undo", () => PHUndo.RestoreState()));
            bindings.Add(new JSONStorableAction("swap arms", () => PHUtils.SwapArms_method()));
            bindings.Add(new JSONStorableAction("swap legs", () => PHUtils.SwapLegs_method()));
            bindings.Add(new JSONStorableAction("toggle focus selected", () => PHUtils.FocusButtonMethod()));
            bindings.Add(new JSONStorableAction("toggle open bone UI", () => PHUtils.SelectButtonMethod()));
        }
        public override void Init()
        {
            foreach(var obj in FindObjectsOfType<Canvas>())
            {
                SuperController.LogMessage(obj.name);
            }
            SuperController.singleton.BroadcastMessage("OnActionsProviderAvailable", this, SendMessageOptions.DontRequireReceiver);
            PHUI.InitPH();
            PHUI.RefreshAtoms();

            RegisterAction(new JSONStorableAction("activate/deactive UI", () => PHUtils.menuUIActive = !PHUtils.menuUIActive));
            RegisterAction(new JSONStorableAction("clear selected bones", () => PHUtils.ClearPressedButtons()));
            RegisterAction(new JSONStorableAction("ik link", () => PHUtils.LinkAll_method()));
            RegisterAction(new JSONStorableAction("reset linking state", () => PHUtils.ResetLink_method()));
            RegisterAction(new JSONStorableAction("reset pose", () => PHUtils.ResetPose_method()));
            RegisterAction(new JSONStorableAction("undo", () => PHUndo.RestoreState()));
            RegisterAction(new JSONStorableAction("swap arms", () => PHUtils.SwapArms_method()));
            RegisterAction(new JSONStorableAction("swap legs", () => PHUtils.SwapLegs_method()));
            RegisterAction(new JSONStorableAction("copy l arm", () => PHUtils.MirrorArmsMethod(false)));
            RegisterAction(new JSONStorableAction("copy r arm", () => PHUtils.MirrorArmsMethod(true)));
            RegisterAction(new JSONStorableAction("copy l leg", () => PHUtils.MirrorLegsMethod(false)));
            RegisterAction(new JSONStorableAction("copy r leg", () => PHUtils.MirrorLegsMethod(true)));
            RegisterAction(new JSONStorableAction("focus Selected bone", () => PHUtils.FocusButtonMethod()));
            RegisterAction(new JSONStorableAction("open bone UI", () => PHUtils.SelectButtonMethod()));

            CreateSpacer(true).height = 20;
            CreateSpacer(false).height = 20;


            activationKb = GenKey("start the motion", KeyCode.Space);
            switchMotionKb = GenKey("switch the motion(position to rotation)", KeyCode.LeftShift);
            switchAxisKb = GenKey("switch the axis", KeyCode.LeftControl);

            CreateSpacer(true).height = 20;
            CreateSpacer(false).height = 20;

            sensetivity = new JSONStorableFloat("sensetivity", 1.0f, 0.1f, 4.0f);
            rotationSpeed = new JSONStorableFloat("rotation Speed", 2.5f, 0.1f, 10.0f);
            sensetivity.storeType = JSONStorableParam.StoreType.Full;
            rotationSpeed.storeType = JSONStorableParam.StoreType.Full;

            CreateSlider(sensetivity, false);
            CreateSlider(rotationSpeed, true);

            RegisterFloat(sensetivity);
            RegisterFloat(rotationSpeed);

            CreateSpacer(true).height = 20;
            CreateSpacer(false).height = 20;

             PHUtils.xPosMultiplier = CreateInverseButton("invert X Pos",false,delegate (bool a) { PHUtils.xPosMultiplier = a?-1.0f:1.0f; });
             PHUtils.yPosMultiplier = CreateInverseButton("invert Y Pos",false,delegate (bool a) { PHUtils.yPosMultiplier = a?-1.0f:1.0f; });
             PHUtils.zPosMultiplier = CreateInverseButton("invert Z Pos",false,delegate (bool a) { PHUtils.zPosMultiplier = a?-1.0f:1.0f; });
             PHUtils.xRotMultiplier = CreateInverseButton("invert X Rot",true ,delegate (bool a) { PHUtils.xRotMultiplier = a?-1.0f:1.0f; });
	         PHUtils.yRotMultiplier = CreateInverseButton("invert Y Rot",true ,delegate (bool a) { PHUtils.yRotMultiplier = a?-1.0f:1.0f; });
             PHUtils.zRotMultiplier = CreateInverseButton("invert Z Rot",true ,delegate (bool a) { PHUtils.zRotMultiplier = a?-1.0f:1.0f; });

            CreateSpacer(true).height = 20;
            CreateSpacer(false).height = 20;

            adjustPosition = new JSONStorableBool("adjust_position", true);
            adjustPosition.storeType = JSONStorableParam.StoreType.Full;
            RegisterBool(adjustPosition);
            CreateToggle(adjustPosition, true);

            alignUpDown = new JSONStorableBool("align_up_down", true);
            alignUpDown.storeType = JSONStorableParam.StoreType.Full;
            RegisterBool(alignUpDown);
            CreateToggle(alignUpDown, false);
        }
        public float CreateInverseButton(string name,bool right,UnityAction<bool> toggleAction)
        {
            JSONStorableBool storableBool = new JSONStorableBool(name,false);
            storableBool.storeType = JSONStorableParam.StoreType.Full;
            RegisterBool(storableBool);
            CreateToggle(storableBool,right);
            storableBool.toggle.onValueChanged.AddListener(toggleAction);
            return storableBool.val?1.0f:-1.0f;
        }
        public Keybind GenKey(string actionName,KeyCode defaultKey)
        {
            CreateButton(actionName, false);
            var btn = CreateButton(defaultKey.ToString(), true);
            Keybind kb = new Keybind(defaultKey,btn);
            keyBindPairs.Add(actionName, kb);
            btn.button.onClick.AddListener(delegate () { keyString = actionName; btn.label = "....";btn.buttonColor = Color.red;isRecording = true; });
            return kb;
        }
        public void FetchKeyOrButton(Keybind kb)
        {
            foreach (KeyCode kcode in Enum.GetValues(typeof(KeyCode)))
            {
                if (Input.GetKeyDown(kcode))
                {
                    kb.Update(kcode);
                    break;
                }
            }
        }
        public class Keybind
        {
            public KeyCode key;
            UIDynamicButton UIbtn;
            public Keybind(KeyCode defaultKey, UIDynamicButton btn)
            {
                key = defaultKey;
                UIbtn = btn;
            }
            public void Update(KeyCode key)
            {
                string keyAsString = key.ToString();
                isRecording = false;
                UIbtn.buttonColor = new Color(0.78f, 0.78f, 0.78f);
                this.UIbtn.label = keyAsString;
                this.key = key;
            }
        }
        private void CheckForKeybindUpdate()
        {
            if(isRecording)
                FetchKeyOrButton(keyBindPairs.Get(keyString));

        }
        private void ApplySymmetry()
        {
            if (PHUtils.isSymmetryToggled)
            {
                PHUtils.MirrorArmsMethod(false);
                PHUtils.MirrorLegsMethod(false);
            }

        }
        void ProccesKeyBindings()
        {

            if (Input.GetKeyDown(activationKb.key))
                PHUtils.PreMoveControllers();

            if (Input.GetKey(activationKb.key))
            {
                float x, y;
                Vector3 moveby;
                x = Input.GetAxis("Mouse X");
                y = Input.GetAxis("Mouse Y");
                moveby = Input.GetKey(switchAxisKb.key) ? new Vector3(0, 0, x + y) : new Vector3(x, y, 0);
                PHUtils.MoveActiveControllers(moveby, !Input.GetKey(switchMotionKb.key));
            }
            if (Input.GetKeyUp(activationKb.key))
                PHUtils.PostMoveControllers();

        }
        void Update()
        {
            ProccesKeyBindings();
            ApplySymmetry();
            CheckForKeybindUpdate();
            if (PHUtils.menuUIActive)
                PHUI.PHMenuSetActive((SuperController.singleton.mainMenuUI.gameObject.activeInHierarchy));
            else
                PHUI.PHMenuSetActive(false);
        }
        public void OnDestroy()
        {

            SuperController.singleton.BroadcastMessage("OnActionsProviderDestroyed", this, SendMessageOptions.DontRequireReceiver);
            PHUI.PHMenuDesroy();
        }
        public void OnDisable()
        {
            PHUI.PHMenuSetActive(false);
        }
        public void OnEnable()
        {
            PHUI.PHMenuSetActive(true);
        }
    }
}
